"use strict";

var _require = require("lodash"),
    kebabCase = _require.kebabCase;

var hasOwn = function hasOwn(obj, key) {
  return Object.prototype.hasOwnProperty.call(obj, key);
};
/**
 * Takes a camelCase string and returns a hyphenated version. Adapted
 * from the hyphenateStyleName module available in the fbjs repo, used by React.
 * In addition to hyphenating the name it also replaces the "ms" prefix with
 * "-ms-" as recommended by Modernizr
 * @param {String} string style attribute name
 * @returns {String} hyphenate style attribute name
 * @see https://modernizr.com/docs/#prefixed
 * @see https://github.com/facebook/fbjs/blob/master/packages/fbjs/src/core/hyphenateStyleName.js
 */
var hyphenateStyleName = function hyphenateStyleName(string) {
  return kebabCase(string).replace(/^ms-/, "-ms-");
};

module.exports = {
  hasOwn,
  hyphenateStyleName
};